% BSFFT - a Matlab toolbox for the butterfly sparse Fourier transform
% 
% Copyright (C) 2012 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2012-08-29 11:09:24 +0200 (Mi, 29. Aug 2012) $
% $Author: melzer $
% $Revision: 3136 $

function [U]=comp_new_func_val1D(alpha,p,N,numSonsLevel_l,l,L,A_Param,B_Param,U_cell,Lleft,Lright)
	%disp(sprintf('number of boxes A in nevel l= %d',size(A_Param,1)))  
	%disp(sprintf('number of Boxex B in level L-l=%d',size(B_Param,1)))

    c2piiN=2*pi*1i/N;
  
	%parameter of the box A and the parent box P of A
	tmpA=A_Param(1,:);
	mA=tmpA(1);
	mP=floor(mA/2);
	
	parent=0;%counter
	
    %initialisation:
	U=zeros(p+2,size(A_Param,1)*size(B_Param,1));
	
	wA=N/2^l;
	wP=N/2^(l-1); %P is parent box of A
    wS=N/2^(L-l+1); %S is a son box of B
	
	for j=1:size(A_Param,1)
        %actuell box A
        tmpA=A_Param(j,:);
		mA=tmpA(1);
		cA=wA*(mA+1/2);	
		xA=cA+alpha.*wA; %Chebyshev grid in A
	
    
		%test, if the parent box changes
        if (  (j~=1) && ( mP~=floor(mA/2)   ))
			parent=parent+1;
			mP=floor(mA/2);
			%disp('different parent box')
			%else disp('same parentbox')
        end
        
        %test, if A is the left or the right son
        if (mod(mA,2)==0)
            Lag=Lleft;
        else
            Lag=Lright;
        end
              
        
		col=parent*numSonsLevel_l+1;
           
		%parameter of parent box P
		cP=wP*(mP+1/2); % center point of the parent box P
		xP=cP+alpha.*wP; %Chebyshev grid in parent box P 
			
		%compute u^{AB} from u^{PS} 
 		for k=1:size(B_Param,1)
			tmpB=B_Param(k,:);
			mB=tmpB(1);
			numSonsB=tmpB(2); %number of sons of B
      
			u_AB=zeros(p,1);
	  
			for r=0:numSonsB-1
                u_PS=U_cell(3:end,col+r); %select u^{P S}
				%box parameters of the sun
                mS=U_cell(2,col+r); 
				cS=wS*(mS+1/2);
				%column vectors of diagonal matrices R^{AS} and S^{PS}
                r_AS=exp(c2piiN*xA*(cS+wS/2)).'; 
				s_PS=exp(-c2piiN*xP*(cS+wS/2)).'; 
				
                tmpu=r_AS.*(Lag*(s_PS.*u_PS));
				u_AB=u_AB+tmpu; 

			end %for
    
			col=col+numSonsB;
      
			U(:,(j-1)*size(B_Param,1)+k)=[mA;mB;u_AB];
		end%for
	end%for
end%function