% BSFFT - a Matlab toolbox for the butterfly sparse Fourier transform
% 
% Copyright (C) 2012 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2012-08-29 11:09:24 +0200 (Mi, 29. Aug 2012) $
% $Author: melzer $
% $Revision: 3136 $

function [U]=comp_new_func_val2D(alpha,p,N,numSonsLevel_l,l,L,A_Param,B_Param,U_cell,Lleft,Lright)
    %disp(sprintf('number of boxes A in nevel l= %d',size(A_Param,1)))  
	%disp(sprintf('number of Boxex B in level L-l=%d',size(B_Param,1)))
 
	c2piiN=2*pi*1i/N;
  
	%parameter of the box A and the parent box P of A
	tmpA=A_Param(1,:);
	m1A=tmpA(1);
	m2A=tmpA(2);
	m1P=floor(m1A/2);
	m2P=floor(m2A/2);

	parent=0; %counter
	%initialisation:
	U=zeros(p^2+4,size(A_Param,1)*size(B_Param,1));
	wA=N/2^l;
    wP=N/2^(l-1); %P is parent box of A
    wS=N/2^(L-l+1); %S is a son box of B
	
	for j=1:size(A_Param,1)
		tmpA=A_Param(j,:);
      m1A=tmpA(1);
		m2A=tmpA(2);
		
      cA=wA*[m1A+1/2,m2A+1/2];	
		xA1=cA(1)+alpha*wA; %Chebyshev grid in A_1
		xA2=cA(2)+alpha*wA; %Chebyshev grid in A_2
	
    
		%test, if the parent box changes
		if (  (j~=1) && ( m1P~=floor(m1A/2)  || (m2P~=floor(m2A/2) ) ))
			parent=parent+1;
			m1P=floor( m1A /2 );
			m2P=floor( m2A /2 );
			%disp('different parent box')
			%else disp('same parentbox')
		end	
		col=parent*numSonsLevel_l+1;
        
        %Lagrange matrices for A_1,A_2
        Lag={zeros(p,p),zeros(p,p)};
        mA=[m1A,m2A];
        for d=1:2
            %test, if A is the left or the right son
            if (mod(mA(d),2)==0)
                Lag{d}=Lleft;
            else
                Lag{d}=Lright;
            end
        end
                 
        %grid in the parent box P of A
		cP=wP*[m1P+1/2,m2P+1/2];
		xP1=cP(1)+alpha*wP; %Chebyshev grid in P^(A_1)
		xP2=cP(2)+alpha*wP; %Chebyshev grid in P^(A_2	)
	
    
		%compute u^{AB} from u^{PS}   
		for k=1:size(B_Param,1)
			%actuell box B
			tmpB=B_Param(k,:);
			m1B=tmpB(1);
			m2B=tmpB(2);
			numSonsB=tmpB(3); %number of sons
      
			U_AB=zeros(p,p);
	  
			for r=0:numSonsB-1
			
				u_PS=U_cell(5:end,col+r); %select u^{PS}
				U_PS=reshape(u_PS,p,p);
				
				%parameter of son boxes
				m1S=U_cell(3,col+r);
				m2S=U_cell(4,col+r);		
				cS=wS*[m1S+1/2,m2S+1/2];
				
				%column vektors of the shift-matrix R^{A_1S_1}, R^{A_2S_2}
				r_A1S1=exp(c2piiN*xA1*(cS(1)+wS/2)).';
				r_A2S2=exp(c2piiN*xA2*(cS(2)+wS/2)).';
				R_AS=r_A1S1*r_A2S2.';
				
            %column vectors of the diagonal matrices S^{P_1S_1},S^{P_2S_2}
				s_P1S1=exp(-c2piiN*xP1*(cS(1)+wS/2)).';
				s_P2S2=exp(-c2piiN*xP2*(cS(2)+wS/2)).';
				S_PS=s_P1S1*s_P2S2.';

				tmpU=S_PS.*U_PS;
            tmpU=Lag{1}*tmpU*Lag{2}.';
				tmpU=R_AS.*tmpU;
			
				U_AB=U_AB+tmpU;
			
			end %for Soehne
  		
		col=col+numSonsB;   
		U(:,(j-1)*size(B_Param,1)+k)=[m1A;m2A;m1B;m2B;U_AB(:)];
		end%for B
	end%for A
end%function
