% BSFFT - a Matlab toolbox for the butterfly sparse Fourier transform
% 
% Copyright (C) 2012 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2012-08-29 11:09:24 +0200 (Mi, 29. Aug 2012) $
% $Author: melzer $
% $Revision: 3136 $

function [F]=comp_new_coeff3D(invG,H,alpha,beta,p,N,numSonsLevel_l,l,L,A_Param,B_Param,F_cell)
	%disp(sprintf('number of boxes A in nevel l= %d',size(A_Param,1)))
	%disp(sprintf('number of Boxex B in level L-l=%d',size(B_Param,1)))
 
	c2piiN=2*pi*1i/N;
  
	%parameter of the box A and the parent box P of A
	tmpA=A_Param(1,:);
	m1A=tmpA(1);
	m2A=tmpA(2);
	m3A=tmpA(3);
	m1P=floor(m1A/2);
	m2P=floor(m2A/2);
	m3P=floor(m3A/2);
  
	parent=0;%counter

	%initialisations:
	wA=N/2^l;
	wB=N/2^(L-l);
	wS=N/2^(L-l+1);
	F=zeros(p^3+6,size(A_Param,1)*size(B_Param,1));
	for j=1:size(A_Param,1)

		%actuell box A
		tmpA=A_Param(j,:);
		m1A=tmpA(1);
		m2A=tmpA(2);
		m3A=tmpA(3);
		cA=wA*[m1A+1/2,m2A+1/2,m3A+1/2];

		xA1=cA(1)+alpha.*wA;%grid in A_1
		xA2=cA(2)+alpha.*wA;%grid in A_2
		xA3=cA(3)+alpha.*wA;%grid in A_3
	

		%test, if the parent box changes
		if (  (j~=1) && ( (m1P~=floor(m1A/2)  || (m2P~=floor(m2A/2)) ) || (m3P~=floor(m3A/2)) ) )
		   parent=parent+1;
		   m1P=floor(m1A/2);
		   m2P=floor(m2A/2);
		   m3P=floor(m3A/2);
		   %disp('different parent box')
		%else disp('same parentbox')
		end
		col=parent*numSonsLevel_l+1;
    
		e2_A1S1=(exp(c2piiN*cA(1)*wS*beta)).';%column vector of diagonal matrix E_2^{A_1 S_1}
		e2_A2S2=(exp(c2piiN*cA(2)*wS*beta)).';%column vector of diagonal matrix E_2^{A_2 S_2}
		e2_A3S3=(exp(c2piiN*cA(3)*wS*beta)).';%column vector of diagonal matrix E_2^{A_3 S_3}
		invd2_A1B1=(exp(-c2piiN*cA(1)*wB*beta)).';%column vector of diagonal matrix (D_2^{A_1 B_1})^{-1}
		invd2_A2B2=(exp(-c2piiN*cA(2)*wB*beta)).';%column vector of diagonal matrix (D_2^{A_2 B_2})^{-1}
		invd2_A3B3=(exp(-c2piiN*cA(3)*wB*beta)).';%column vector of diagonal matrix (D_2^{A_3 B_3})^{-1}

		%tensors
		invD2_AB=zeros(p,p,p);
		E2_AB=zeros(p,p,p); 
		for t=1:p      
			invD2_AB(:,:,t)=invd2_A3B3(t)*(invd2_A1B1*invd2_A2B2.');
			E2_AB(:,:,t)=e2_A3S3(t)*(e2_A1S1*e2_A2S2.');
		end


		%compute f^{AB} from f^{PS} 
		for k=1:size(B_Param,1)
			%actuell box B
			tmpB=B_Param(k,:);
			m1B=tmpB(1);
			m2B=tmpB(2);
			m3B=tmpB(3);
			cB=wB*[m1B+1/2,m2B+1/2,m3B+1/2];
			numSonsB=tmpB(4);%number of sons

			tensorU_AB=zeros(p,p,p);

			for r=0:numSonsB-1

				tmpf=F_cell(7:end,col+r); %select f^(P S)
				tensorF_AB=reshape(tmpf,p,p,p);

				%box parameters of the son
				m1S=F_cell(4,col+r); 
				m2S=F_cell(5,col+r);
				m3S=F_cell(6,col+r);
				cS=wS*[m1S+1/2,m2S+1/2,m3S+1/2];


				c_A1S1=( exp(c2piiN*xA1.*(cS(1)-cB(1))) ).'; %column vector of diagonal matrix C^{A_1 S_1}
				c_A2S2=( exp(c2piiN*xA2.*(cS(2)-cB(2))) ).'; %column vector of diagonal matrix C^{A_2 S_2} 
				c_A3S3=( exp(c2piiN*xA3.*(cS(3)-cB(3))) ).'; %column vector of diagonal matrix C^{A_3 S_3}   

				C_AS=zeros(p,p,p);
				for t=1:p
					   C_AS(:,:,t)=c_A3S3(t)*(c_A1S1*c_A2S2.');
				end%for

				tensorU=E2_AB.*tensorF_AB;
				tensorU=nmode3D_coeff(tensorU,H,p);
				tensorU=C_AS.*tensorU;			  
				tensorU_AB=tensorU_AB+tensorU;

				end%for sons
				
			col=col+numSonsB;
			
			tensorU_AB=nmode3D_coeff(tensorU_AB,invG,p);	
			tensorU_AB=invD2_AB.*tensorU_AB;
			f_AB=tensorU_AB(:);
			tmp=[m1A;m2A;m3A;m1B;m2B;m3B;f_AB];
			F(:,(j-1)*size(B_Param,1)+k)=tmp;

	end%for B
end%for A

end%function