% BSFFT - a Matlab toolbox for the butterfly sparse Fourier transform
% 
% Copyright (C) 2012 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2012-08-29 11:09:24 +0200 (Mi, 29. Aug 2012) $
% $Author: melzer $
% $Revision: 3136 $

function u=mtimes(plan,f)
% computing the butterfly sparse Fourier transform
%
% u=mtimes(plan,f)
%
% INPUT
%   plan  object of class sparse_FFT1D
%   f     complex column vector of length size(plan.MOmega,1)
%
% OUTPUT
%   u  complex column vector of length  size(plan.MX,1)

    L=log2(plan.N);
    %Box A=A_1 x A_2 x A_3 x A_4
    cA=[plan.N/2,plan.N/2,plan.N/2,plan.N/2];
    wA=plan.N;
	xA1=cA(1)+plan.alpha*wA; %grid in A_1
	xA2=cA(2)+plan.alpha*wA; %grid in A_2
    xA3=cA(3)+plan.alpha*wA; %grid in A_3
	xA4=cA(4)+plan.alpha*wA; %grid in A_4
	m1A=0;
    m2A=0;
	m3A=0;
	m4A=0;
	
    B_Boxes=plan.LastlevelB;

    switch(plan.option)
        case 'Mtb'
            %Computing coefficient vectors for l=0, A=X_00000
            F_cell=zeros(plan.p^4+8,size(B_Boxes,1));
            wB=1;
            for j=1:size(B_Boxes,1)
                % actuell box B B=B_1 x B_2 x B_3 x B_4
                tmp=B_Boxes(j,:);
                m1B=tmp{1};
                m2B=tmp{2};
                m3B=tmp{3};
                m4B=tmp{4};
                B_index=tmp{5};
                cB=[m1B+1/2,m2B+1/2,m3B+1/2,m4B+1/2];
                
                %computing the function values of the exact functions u^B
                u_AB=comp_first_func_val4D(plan.MOmega,plan.p,plan.N,xA1,xA2,xA3,xA4,B_index,f);
                %computing the function values of the exact functions u^B
                f_AB=comp_first_coeff4D(plan.invG,plan.beta,plan.p,plan.N,xA1,xA2,xA3,xA4,cA,cB,wB,u_AB);
                
                tmp=[m1A;m2A;m3A;m4A;m1B;m2B;m3B;m4B;f_AB];
                F_cell(:,j)=tmp;
            end%for j
            
            %Computing new function values from recursive from the previous level
            for l=1:L
                l
                numSonsLevel_l=plan.numSonsLevel(l);
                A_Param=plan.ABoxes{l};
                B_Param=plan.BBoxes{l};
                F_cell=comp_new_coeff4D(plan.invG,plan.H,plan.alpha,plan.beta,plan.p,plan.N,numSonsLevel_l,l,L,A_Param,B_Param,F_cell);
            end%for l
            
            %last step: evaluation in the nodes MX
            cB=[plan.N/2 plan.N/2 plan.N/2 plan.N/2];
            wB=plan.N;
            xiB1=cB(1)+plan.beta*wB;
            xiB2=cB(2)+plan.beta*wB;
            xiB3=cB(3)+plan.beta*wB;
            xiB4=cB(4)+plan.beta*wB;
            c2piiN=2*pi*1i/plan.N;
            M1=size(plan.MX,1);
            u=zeros(M1,1);
            A_Boxes=plan.LastlevelA; %boxes A of level L
            
            for r=1:size(F_cell,2)
                tmp=F_cell(:,r);
                f_AB=tmp(9:end);
                tensorF_AB=reshape(f_AB,plan.p,plan.p,plan.p,plan.p);
                tmpA=A_Boxes(r,:);
                A_index=tmpA{5};
                
                for k=1:length(A_index)
                    d1=exp(c2piiN*plan.MX(A_index(k),1)*xiB1).';
                    d2=exp(c2piiN*plan.MX(A_index(k),2)*xiB2).';
                    d3=exp(c2piiN*plan.MX(A_index(k),3)*xiB3).';
                    d4=exp(c2piiN*plan.MX(A_index(k),4)*xiB4).';
                    D=zeros(plan.p,plan.p,plan.p,plan.p);
                    for s1=1:plan.p
                        for s2=1:plan.p
                            D(:,:,s2,s1)=d4(s1)*d3(s2)*(d1*d2.');
                        end%for s2
                    end%for s1
                    tmpF=D.*tensorF_AB;
                    u(A_index(k))=sum(tmpF(:));
                end%for k
                
            end%for r
            
        case {'Ltb','Ltb*'}
            %Computing function values for l=0, A=X_0000
            U_cell=zeros(plan.p^4+8,size(B_Boxes,1));
            for j=1:size(B_Boxes,1)
                
                % actuell box B B=B_1 x B_2 x B_3 x B_4
                tmp=B_Boxes(j,:);
                m1B=tmp{1};
                m2B=tmp{2};
                m3B=tmp{3};
                m4B=tmp{4};
                B_index=tmp{5};
                
                u_B=comp_first_func_val4D(plan.MOmega,plan.p,plan.N,xA1,xA2,xA3,xA4,B_index,f);
                tmp=[m1A;m2A;m3A;m4A;m1B;m2B;m3B;m4B;u_B];
                U_cell(:,j)=tmp;
            end%for j
            
            %Computing new function values from recursive from the previous level
            for l=1:L
                l
                numSonsLevel_l=plan.numSonsLevel(l);
                A_Param=plan.ABoxes{l};
                B_Param=plan.BBoxes{l};
                U_cell=comp_new_func_val4D(plan.alpha,plan.p,plan.N,numSonsLevel_l,l,L,A_Param,B_Param,U_cell,plan.Lleft,plan.Lright);
            end%for
            
            
            %last step: evaluation in the nodes MX
            cB=[plan.N/2,plan.N/2,plan.N/2,plan.N/2];
            wB=plan.N;
            wA=1;
            c2piiN=2*pi*1i/plan.N;
            M1=size(plan.MX,1);
            u=zeros(M1,1);
            A_Boxes=plan.LastlevelA; %A-Boxes of level L
           
            
            switch(plan.option)
                case 'Ltb'
                    denom=zeros(plan.p,1); %denominator of lagrange polynomial
                    zalpha=exp(-2*pi*1i*plan.alpha/(plan.p-1));
                    for s=1:plan.p
                        denom(s)=1;
                        for j=[1:s-1,s+1:plan.p]
                            denom(s)=denom(s)*(zalpha(s)-zalpha(j));
                        end
                    end
                    
                    c2piiNpwB=2*pi*1i*wB/((plan.p-1)*plan.N);
                    unitp=ones(plan.p-1,1);
                    for r=1:size(U_cell,2)
                        tmp=U_cell(:,r);
                        m1A=tmp(1);
                        m2A=tmp(2);
                        m3A=tmp(3);
                        m4A=tmp(4);
                        
                        u_AB=tmp(9:end);
                        U_AB=reshape(u_AB,plan.p,plan.p,plan.p,plan.p);
                        
                        tmpA=A_Boxes(r,:);
                        A_index=tmpA{5};
                        
                        cA=wA*[m1A+1/2,m2A+1/2,m3A+1/2,m4A+1/2];
                        xA1=cA(1)+plan.alpha*wA;
                        xA2=cA(2)+plan.alpha*wA;
                        xA3=cA(3)+plan.alpha*wA;
                        xA4=cA(4)+plan.alpha*wA;
                        
                        Mat_x=plan.MX(A_index,:);
                        numA=length(A_index);
                        Mat_cA=repmat(cA,numA,1);
                        Mat_z=exp(-c2piiNpwB*(Mat_x-Mat_cA));
                        
                        tmpz1=unitp*(Mat_z(:,1)).';
                        tmpz1=tmpz1(:);
                        
                        tmpz2=unitp*(Mat_z(:,2)).';
                        tmpz2=tmpz2(:);
                        
                        tmpz3=unitp*(Mat_z(:,3)).';
                        tmpz3=tmpz3(:);
                        
                        tmpz4=unitp*(Mat_z(:,4)).';
                        tmpz4=tmpz4(:);
                        
                        unitA=ones(1,numA);
                        
                        
                        lag1=zeros(numA,plan.p);
                        lag2=zeros(numA,plan.p);
                        lag3=zeros(numA,plan.p);
                        lag4=zeros(numA,plan.p);
                        for s=1:plan.p
                            tmpzalpha=zalpha([1:s-1,s+1:plan.p]);
                            tmp=(tmpzalpha.')*unitA;
                            tmp=tmp(:);
                            
                            numer1=reshape(tmpz1-tmp,plan.p-1,numA);
                            numer1=prod(numer1,1);%numerator of 1st lagrange polynomial, row-vector
                            
                            numer2=reshape(tmpz2-tmp,plan.p-1,numA);
                            numer2=prod(numer2,1);%numerator of 1st lagrange polynomial, row vector
                            
                            numer3=reshape(tmpz3-tmp,plan.p-1,numA);
                            numer3=prod(numer3,1);%numerator of 1st lagrange polynomial, row vector
                            
                            numer4=reshape(tmpz4-tmp,plan.p-1,numA);
                            numer4=prod(numer4,1);%numerator of 1st lagrange polynomial,row vector
                            
                            
                            
                            lag1(:,s)=numer1.'/denom(s);
                            lag2(:,s)=numer2.'/denom(s);
                            lag3(:,s)=numer3.'/denom(s);
                            lag4(:,s)=numer4.'/denom(s);
                        end
                        
                        E=exp(c2piiN*Mat_x*(cB+wB/2)');
                        d1=exp(-c2piiN*xA1*(cB(1)+wB/2)).';
                        d2=exp(-c2piiN*xA2*(cB(2)+wB/2)).';
                        d3=exp(-c2piiN*xA3*(cB(3)+wB/2)).';
                        d4=exp(-c2piiN*xA4*(cB(4)+wB/2)).';
                        for k=1:numA
                            l_1=d1.*(lag1(k,:)).';
                            l_2=d2.*(lag2(k,:)).';
                            l_3=d3.*(lag3(k,:)).';
                            l_4=d4.*(lag4(k,:)).';
                            D=zeros(plan.p,plan.p,plan.p,plan.p);
                            for s1=1:plan.p
                                for s2=1:plan.p
                                    D(:,:,s2,s1)=l_4(s1)*l_3(s2)*(l_1*l_2.');
                                end%for s2
                            end%for s1
                            tmpU=D.*U_AB;
                            u(A_index(k))=E(k)*sum(tmpU(:));
                        end%for k
                        
                    end%for r
                    
                case 'Ltb*'
                    Lagrange=plan.Laglast;
                    M1=size(plan.MX,1);
                    L1=Lagrange(1:M1,:);
                    L2=Lagrange(M1+1:2*M1,:);
                    L3=Lagrange(2*M1+1:3*M1,:);
                    L4=Lagrange(3*M1+1:end,:);
                    
                    for r=1:size(U_cell,2)
                        tmp=U_cell(:,r);
                        m1A=tmp(1);
                        m2A=tmp(2);
                        m3A=tmp(3);
                        m4A=tmp(4);
                        
                        u_AB=tmp(9:end);
                        U_AB=reshape(u_AB,plan.p,plan.p,plan.p,plan.p);
                        
                        tmpA=A_Boxes(r,:);
                        A_index=tmpA{5};
                        
                        cA=wA*[m1A+1/2,m2A+1/2,m3A+1/2,m4A+1/2];
                        xA1=cA(1)+plan.alpha*wA;
                        xA2=cA(2)+plan.alpha*wA;
                        xA3=cA(3)+plan.alpha*wA;
                        xA4=cA(4)+plan.alpha*wA;
                        
                        Mat_x=plan.MX(A_index,:);
                        E=exp(c2piiN*Mat_x*(cB+wB/2)');
                        d1=exp(-c2piiN*xA1*(cB(1)+wB/2)).';
                        d2=exp(-c2piiN*xA2*(cB(2)+wB/2)).';
                        d3=exp(-c2piiN*xA3*(cB(3)+wB/2)).';
                        d4=exp(-c2piiN*xA4*(cB(4)+wB/2)).';
                        for k=1:length(A_index)
                            l_1=d1.*(L1(A_index(k),:)).';
                            l_2=d2.*(L2(A_index(k),:)).';
                            l_3=d3.*(L3(A_index(k),:)).';
                            l_4=d4.*(L4(A_index(k),:)).';
                            D=zeros(plan.p,plan.p,plan.p,plan.p);
                            for s1=1:plan.p
                                for s2=1:plan.p
                                    D(:,:,s2,s1)=l_4(s1)*l_3(s2)*(l_1*l_2.');
                                end%for s2
                            end%for s1
                            tmpU=D.*U_AB;
                            u(A_index(k))=E(k)*sum(tmpU(:));
                        end%for k
                        
                    end%for r
                    
                case 'old'
                    denom=zeros(plan.p,1); %denominator of lagrange polynomial
                    zalpha=exp(-2*pi*1i*plan.alpha/(plan.p-1));
                    for s=1:plan.p
                        denom(s)=1;
                        for j=[1:s-1,s+1:plan.p]
                            denom(s)=denom(s)*(zalpha(s)-zalpha(j));
                        end
                    end
                    
                    c2piiNpwB=2*pi*1i*wB/((plan.p-1)*plan.N);
                    for r=1:size(U_cell,2)
                        tmp=U_cell(:,r);
                        m1A=tmp(1);
                        m2A=tmp(2);
                        m3A=tmp(3);
                        m4A=tmp(4);
                        
                        u_AB=tmp(9:end);
                        U_AB=reshape(u_AB,plan.p,plan.p,plan.p,plan.p);
                        
                        tmpA=A_Boxes(r,:);
                        A_index=tmpA{5};
                        
                        cA=wA*[m1A+1/2,m2A+1/2,m3A+1/2,m4A+1/2];
                        xA1=cA(1)+plan.alpha*wA;
                        xA2=cA(2)+plan.alpha*wA;
                        xA3=cA(3)+plan.alpha*wA;
                        xA4=cA(4)+plan.alpha*wA;
                        
                        Mat_x=plan.MX(A_index,:);
                        numA=length(A_index);
                        Mat_cA=repmat(cA,numA,1);
                        Mat_z=exp(-c2piiNpwB*(Mat_x-Mat_cA));
                        lag1=zeros(numA,plan.p);
                        lag2=zeros(numA,plan.p);
                        lag3=zeros(numA,plan.p);
                        lag4=zeros(numA,plan.p);
                        for s=1:plan.p
                            numer1=1;
                            numer2=1;
                            numer3=1;
                            numer4=1;
                            for j=[1:s-1,s+1:plan.p]
                                numer1=numer1.*(Mat_z(:,1)-zalpha(j)); %numerator of 1st lagrange polynomial
                                numer2=numer2.*(Mat_z(:,2)-zalpha(j)); %numerator of 2nd lagrange polynomial
                                numer3=numer3.*(Mat_z(:,3)-zalpha(j)); %numerator of 3rd lagrange polynomial
                                numer4=numer4.*(Mat_z(:,4)-zalpha(j)); %numerator of 4th lagrange polynomial
                            end
                            lag1(:,s)=numer1/denom(s);
                            lag2(:,s)=numer2/denom(s);
                            lag3(:,s)=numer3/denom(s);
                            lag4(:,s)=numer4/denom(s);
                        end
                        
                        E=exp(c2piiN*Mat_x*(cB+wB/2)');
                        d1=exp(-c2piiN*xA1*(cB(1)+wB/2)).';
                        d2=exp(-c2piiN*xA2*(cB(2)+wB/2)).';
                        d3=exp(-c2piiN*xA3*(cB(3)+wB/2)).';
                        d4=exp(-c2piiN*xA4*(cB(4)+wB/2)).';
                        for k=1:numA
                            l_1=d1.*(lag1(k,:)).';
                            l_2=d2.*(lag2(k,:)).';
                            l_3=d3.*(lag3(k,:)).';
                            l_4=d4.*(lag4(k,:)).';
                            D=zeros(plan.p,plan.p,plan.p,plan.p);
                            for s1=1:plan.p
                                for s2=1:plan.p
                                    D(:,:,s2,s1)=l_4(s1)*l_3(s2)*(l_1*l_2.');
                                end%for s2
                            end%for s1
                            tmpU=D.*U_AB;
                            u(A_index(k))=E(k)*sum(tmpU(:));
                        end%for k
                        
                    end%for r
            end%switch
                
            
    end%switch
    
  
  end%mtimes