% BSFFT - a Matlab toolbox for the butterfly sparse Fourier transform
% 
% Copyright (C) 2012 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2012-08-29 13:13:15 +0200 (Mi, 29. Aug 2012) $
% $Author: melzer $
% $Revision: 3139 $

%This script computes the relative error in one box for different represantations 
%with rescpect to the local expansion degree p.

L=10;
%L=14;
l=5;
savefile=sprintf('../data/relative_error1D_box_L%d.mat',L);

%B=\Omega_{L-l,m_Omega,n_Omega} 
%parameters
N=2^L;
%box A:=[N/2^l*mA , N/2^l*(mA+1)] 
mA=0; % 0<= mA,nA<=2^l-1 
%box B:=[N/2^(L-l)*mB , N/2^(L-l)*(mB+1)] 
mB=0; % 0 <= mB,nB <=2^(L-l)-1

M1=N;
M2=N;

%random nodes:
MX=N/(2^l)*mA+N/(2^l)*rand(M1,1);
MOmega=N/(2^(L-l))*mB+N/(2^(L-l))*rand(M2,1);


%box parameters
cA=N/2^l*(mA+1/2); 
cB=N/2^(L-l)*(mB+1/2);
wA=N/2^l;
wB=N/2^(L-l);

pmin=3;
pmax=16;



E_M=inf*ones(pmax-pmin,5);
E_Mpseud=inf*ones(pmax-pmin,5);
%E_pot=inf*ones(pmax-pmin,5);

	
F=exp(2*pi*1i*MX*MOmega'/N); %exact matrix


%represantation by Monomial-type basis with different nodes in A and B
%first parameter: grid in A
%second parameter: grid in B
%parameters:	
	%t=chebyshev grid
	%tmax extremal points of T_p
	%h equispaces grid
err_tt=[];
err_tmaxtmax=[];
err_tmaxh=[];
err_th=[];
err_hh=[];

%representation by Lagrange-type basis
err_lag=[];

%condition number of matrix G
k=[];

for p=pmin:pmax
	j=0:1:p-1;
	t=1/2*cos((2*j+1)./(2*p)*pi);%alpha
	tmax=1/2*cos(j*pi/(p-1));%beta
	h=1/2-j./(p-1);
	G=exp(2*pi*1i*h'*t);
	k=[k;cond(G,2)];

	disp(sprintf('p= %d',p))

	F_tt=approxF(MX,MOmega,p,N,cA,cB,wA,wB,t,t);
	F_tmaxtmax=approxF(MX,MOmega,p,N,cA,cB,wA,wB,tmax,tmax);
	F_tmaxh=approxF(MX,MOmega,p,N,cA,cB,wA,wB,tmax,h);
	F_th=approxF(MX,MOmega,p,N,cA,cB,wA,wB,t,h);
	F_hh=approxF(MX,MOmega,p,N,cA,cB,wA,wB,h,h);

	%compare errors by norm max_{i,j} |F_alphabeta(i,j)-F(i,j)| 	
	tmp=abs(F_tt-F);
	err_tt=[err_tt;max(tmp(:))];

	tmp=abs(F_tmaxtmax-F);
	err_tmaxtmax=[err_tmaxtmax;max(tmp(:))];

	tmp=abs(F_tmaxh-F);
	err_tmaxh=[err_tmaxh;max(tmp(:))];

	tmp=abs(F_th-F);
	err_th=[err_th;max(tmp(:))];

	tmp=abs(F-F_hh);
	err_hh=[err_hh;max(tmp(:))];
		
	%representation by Lagrange-type basis%
	c2piiNpwB=2*pi*1i*wB/((p-1)*N);
          
    xA_node=cA+wA*t;
    c2piiN=2*pi*1i/N;
    
	F1=exp(c2piiN*xA_node'*MOmega');
    d1=exp(c2piiN*(cB+wB/2)*MX);
    d2=exp(-c2piiN*(cB+wB/2)*xA_node');
    
	alpha=t;
	Lag=zeros(M1,p);
	znode=exp(-2*pi*1i*alpha/(p-1));
    MZ=exp(-c2piiNpwB*(MX-cA));	
	for s=1:p
            Lag(:,s)=lagpol_z(p,MZ,znode,s);
	end
		
    F_lag=diag(d1)*Lag*diag(d2)*F1;
    
    tmp=abs(F_lag-F);
    err_lag=[err_lag;max(tmp(:))];

	save(savefile,'pmin','pmax','k','err_tt','err_tmaxtmax','err_tmaxh','err_th','err_hh','err_lag','N','l')
end%for