% BSFFT - a Matlab toolbox for the butterfly sparse Fourier transform
% 
% Copyright (C) 2012 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2012-08-29 13:13:15 +0200 (Mi, 29. Aug 2012) $
% $Author: melzer $
% $Revision: 3139 $

%This script plots the condition number of the Vandermonde Matrix
%(interpolation matrix) G, which is used in the Monomial-type basis
%algorithm.

%see figure 4.2. (a) in the paper

clear all;

pmin=3;
pmax=16;

k=[];

for p=pmin:pmax
	j=(0:1:p-1);
	beta=(1/2-j/(p-1))';
	alpha=1/2*cos( (2*j+1)/(2*p)*pi )';
	G=exp(2*pi*1i*alpha*beta');
	k=[k;cond(G,2)];
end

vec_p=pmin:pmax;
s1=sqrt(vec_p).*((vec_p-1)/(2*pi)).^(vec_p-1);
s2=2.^(vec_p-1)./sqrt(vec_p).*((vec_p-1)/pi).^(vec_p-1);

h(1)=semilogy(vec_p,k,'d-');
hold on;
h(2)=semilogy(vec_p,s1,'r-.');
h(3)=semilogy(vec_p,s2,'r--');


set(h(1),'Color',[0,1/2,0]);%green

set(h,'linewidth',2,'MarkerSize',8);

fs=14;
set(gca,'Fontsize',fs);
set(get(gca,'title'),'Fontsize',fs);
set(get(gca,'XLabel'),'Fontsize',fs);
set(get(gca,'YLabel'),'Fontsize',fs);
hl=legend('$\kappa(\mathbf{M}^{AB})$','Thm. 2.7, (2.33)  ','Thm, 2.7, (2.34)','Location','NorthWest');
set(hl,'interpreter','latex');

xlabel('p')
axis([vec_p(1),vec_p(end),1,max(k)]);
set(gca,'YTick',10.^(0:1:18))
set(gca,'XTick',pmin:pmax)
saveas(gcf,'../pics/condition_number_G','png')