% BSFFT - a Matlab toolbox for the butterfly sparse Fourier transform
% 
% Copyright (C) 2012 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2012-08-29 13:13:15 +0200 (Mi, 29. Aug 2012) $
% $Author: melzer $
% $Revision: 3139 $

%This script plots the condition number of Lagrange matrix, which is used in the Lagrange-type basis
%algorithm.

%see figure 4.2. (b) in the paper

clear all;
pmin=3;
pmax=16;

kleft=[]; %condition number of L^A for a left box A
%kright=[];
c=[];


for p=pmin:pmax
	
	j=(0:1:p-1);
	alpha=1/2*cos( (2*j+1)/(2*p)*pi )';
	
	Lleft=zeros(p,p);
	Lright=zeros(p,p);
	for s=1:p
		Lleft(:,s)=lagpol_alpha(0,p,alpha,s);
        Lright(:,s)=lagpol_alpha(1,p,alpha,s);
	end

	kleft=[kleft;cond(Lleft,2)];
	%kright=[kright;cond(Lright,2)];
	
	K_p=( 2*pi^2/((1-cos(2*pi/(p-1)))*(p-1)^2) )^(p-1)*sqrt(2*p*34^p)/(4);
	c=[c;K_p];

end


vec_p=pmin:pmax;
 
h=semilogy(vec_p,kleft,'bd-',vec_p,c,'r--');
set(h,'linewidth',2,'MarkerSize',8);
%axis([pmin-0.05,pmax,min( [min([kleft;c]),10^(-1)]), max([max([kleft;c]),10^(18)])]);
axis([vec_p(1),vec_p(end),1,10^(17)]);

fs=14;
set(gca,'Fontsize',fs);
set(get(gca,'title'),'Fontsize',fs);
set(get(gca,'XLabel'),'Fontsize',fs);
set(get(gca,'YLabel'),'Fontsize',fs);
set(gca,'YTick',10.^(0:1:17))
set(gca,'XTick',pmin:pmax)

hl=legend('$\kappa(\mathbf{L}^{A})$','Thm. 2.8','Location','NorthWest');
set(hl,'interpreter','latex');
xlabel('p')
saveas(gcf,'../pics/condition_number_Lag','png')