% BSFFT - a Matlab toolbox for the butterfly sparse Fourier transform
% 
% Copyright (C) 2012 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2012-08-29 13:13:15 +0200 (Mi, 29. Aug 2012) $
% $Author: melzer $
% $Revision: 3139 $



%This script computes the time of the butterfly sparse FFT for dimension
%d=1 with respect to the nonharmonic bandwith N and problem sizes M1,M2.
%The important computations will be saved in a mat file in the
%directory data, see the initialisation of savefile.
%For plotting you have to call the function show1D_times.

clear all;
addpath('..');
addpath('utilities')
addpath('data')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%parameters: 
p=4;  %figure 4.5. (a)
%p=8; %figure 4.5. (b)
minL=1;
maxL=12;
savefile=sprintf('data/times1D_p%.mat',p); %File in das gespeichert wird
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

d=1; %dimension

display(sprintf('test for p=%d',p))

vec_L=[];
t_Mtb=[];
t_Ltb=[];
t_Ltbstar=[];
t_ex=[];
err_Mtb=[];
err_Ltb=[];
err_Ltbstar=[];

for L=minL:maxL
	display('=========================================================')
	display(L)
	N=2^L;
	vec_L=[vec_L;L];

	%nodes:
	M1=N;
	M2=N;
	%matrix MX
	MX=N*rand(M1,1);
	%matrix M_Omega
	MOmega=N*rand(M2,1);
	f=complex(rand(M2,1)-1/2,rand(M2,1)-1/2);
	
    display('Monomial-type basis')
	plan1=sparse_FFT1D(MX,MOmega,p,N,'Mtb');	
	tic
		u_Mtb=mtimes(plan1,f);
		t_Mtb=[t_Mtb;toc];
	clear plan_coeff;
    
	display('Lagrange-type basis')
	plan2=sparse_FFT1D(MX,MOmega,p,N,'Ltb');
	tic
		u_Ltb=mtimes(plan2,f);
		t_Ltb=[t_Ltb;toc];
	clear plan2;		
	
	display('Lagrange-type basis with precomputation of the Lagrange polynomials in the nodes MX')
	plan3=sparse_FFT1D(MX,MOmega,p,N,'Ltb*');
	tic
		u_Ltbstar=mtimes(plan3,f);
		t_Ltbstar=[t_Ltbstar;toc];
	clear plan3;
		
	
		
	display('exact computation')
	tic
		u_ex=u_exact(MX,MOmega,f,N);
		t_ex=[t_ex;toc];
	
	display('relative errors:')
		err1=norm(u_ex-u_Mtb,inf)/norm(f,1)
		err2=norm(u_ex-u_Ltb,inf)/norm(f,1)
		err3=norm(u_ex-u_Ltbstar,inf)/norm(f,1)
        err_Mtb=[err_Mtb;err1];
        err_Ltb=[err_Ltb;err2];
		err_Ltbstar=[err_Ltbstar;err3];
		
		
	
	%save computations in savefile
	%save(savefile,'p','vec_L','t_ex','t_fast','t_normal','t_coeff','err_fast','err_normal','err_coeff')
    save(savefile,'p','vec_L','t_ex','t_Mtb','t_Ltb','t_Ltbstar','err_Mtb','err_Ltb','err_Ltbstar','d')
end

