% BSFFT - a Matlab toolbox for the butterfly sparse Fourier transform
% 
% Copyright (C) 2012 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2012-08-29 13:13:15 +0200 (Mi, 29. Aug 2012) $
% $Author: melzer $
% $Revision: 3139 $

%This script computes the time of the butterfly sparse FFT for dimension
%d=4 with respect to the nonharmonic bandwith N and problem sizes M1,M2.
%The important computations will be saved in a mat file in the
%directory data, see the initialisation of savefile.
%For plotting you have to call the function show4D_times.


clear all;
addpath('..');
addpath('utilities')
addpath('data')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%set parameters
p=4;    %figure 4.6. (e)
%p=8;   %figure 4.6. (f)
minL=1;
maxL=9;
savefile=sprintf('data/times4D_hyperebene_p%d.mat',p); 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

d=4; %dimension

display(sprintf('Test fuer p=%d',p))
vec_L=[];
t_Mtb=[];
t_Ltb=[];
t_Ltbstar=[];
t_ex=[];
err_Mtb=[];
err_Ltb=[];
err_Ltbstar=[];


for L=minL:maxL
		display('=========================================================')
		display(L)
		N=2^L;
		vec_L=[vec_L;L];
		
		Ntilde=floor(N/sqrt(3));
		
		%nodes on hyperplanes in [0,N]^4
		k=0:1:N-1;
		[K1,K2,K3]=meshgrid(k,k,k);
		K1=K1(:);
		K2=K2(:);
		K3=K3(:);
		%matrix M_Omega
		MOmega=[K1,K1,K2,K3];
		M2=size(MOmega,1);
		display('number of sample nodes in Omega')
		M2
		%matrix M_X
		MX=[K1,K2,K3,K3];
		M1=size(MX,1);
		display('number of sample nodes in X');
		M1
		clear K1 K2 K3;
		%fourier coefficients
		f=complex(rand(M2,1)-1/2,rand(M2,1)-1/2);
		
	
	
	display('Monomial-type basis')
	plan1=sparse_FFT1D(MX,MOmega,p,N,'Mtb');	
	tic
		u_Mtb=mtimes(plan1,f);
		t_Mtb=[t_Mtb;toc];
	clear plan_coeff;
    
	display('Lagrange-type basis')
	plan2=sparse_FFT1D(MX,MOmega,p,N,'Ltb');
	tic
		u_Ltb=mtimes(plan2,f);
		t_Ltb=[t_Ltb;toc];
	clear plan2;		
	
	display('Lagrange-type basis with precomputation of the Lagrange polynomials in the nodes MX')
	plan3=sparse_FFT1D(MX,MOmega,p,N,'Ltb*');
	tic
		u_Ltbstar=mtimes(plan3,f);
		t_Ltbstar=[t_Ltbstar;toc];
	clear plan3;	
		
	if (L<5)
	display('exact computation')
	tic
		u_ex=u_exact(MX,MOmega,f,N);
		t_ex=[t_ex;toc];
	
	display('relative errors:')
		err1=norm(u_ex-u_Mtb,inf)/norm(f,1)
		err2=norm(u_ex-u_Ltb,inf)/norm(f,1)
		err3=norm(u_ex-u_Ltbstar,inf)/norm(f,1)
        err_Mtb=[err_Mtb;err1];
        err_Ltb=[err_Ltb;err2];
		err_Ltbstar=[err_Ltbstar;err3];
    end
	
	clear u_Mtb u_ex u_Ltb;
	%save computations
	save(savefile,'p','vec_L','t_ex','t_Mtb','t_Ltb','t_Ltbstar','err_Mtb','err_Ltb','err_Ltbstar','d')

end