% BSFFT - a Matlab toolbox for the butterfly sparse Fourier transform
% 
% Copyright (C) 2012 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2012-08-29 11:15:41 +0200 (Mi, 29. Aug 2012) $
% $Author: melzer $
% $Revision: 3137 $

%START1D test script for d=1
clear all;
addpath('paper_KuMe2012/utilities')

%setting parameters:
p=7; %approximation rank

L=7; %number of levels
N=2^L; %defining domains, X=Omega=[0,N]

M1=N; %number of nodes in X
M2=N; %number of nodes in Omega

MX=[0;N*rand(M1-1,1)]; %nodes in X
MOmega=N*rand(M2,1);   %nodes in Omega  

f=ones(M2,1); %coefficients


%plans for different variants

%1. 
disp('Mtb: computation via monomial-type basis')
plan1=sparse_FFT1D(MX,MOmega,p,N,'Mtb'); %monomial type basis
     start1=tic;
 u1=mtimes(plan1,f);
     t1=toc(start1);
 clear plan1;
 	 
%2. 
disp('Ltb: computation via Lagrange-type basis without precomputation of the polynomials in the last step') 
plan2=sparse_FFT1D(MX,MOmega,p,N,'Ltb'); 
	start2=tic;
u2=mtimes(plan2,f);
	t2=toc(start2);
clear plan2;

%3. 
disp('Ltb*: computation via Lagrange type basis with precomputation of the polynomials in the last step')
plan3=sparse_FFT1D(MX,MOmega,p,N,'Ltb*');
    start3=tic;
u3=mtimes(plan3,f);
	t3=toc(start3);
clear plan3;

%4. 
display('exact computation')
tic
    u_ex=u_exact(MX,MOmega,f,N);
t_ex=toc;


fprintf('\n')
disp(sprintf('test with parameters:'))
disp(sprintf('  N= %d',N))
disp(sprintf('  M1= %d',M1))
disp(sprintf('  M2= %d',M2))
disp(sprintf('  p= %d',p))
disp(sprintf('  time of the exact computation = %d', t_ex))

fprintf('\n')
%time
disp(sprintf('comparison of the three variants:'))
disp(sprintf('  time of Mtb  = %d',t1))
disp(sprintf('  time of Ltb  = %d',t2))
disp(sprintf('  time of Ltb* = %d',t3))
disp('-----------------------------------------------');

%absolute error
disp(sprintf('  Mtb:  max_i |u_{ex}(i) - u_{approx}(i)|= %d',max(abs(u_ex-u1))))
disp(sprintf('  Ltb:  max_i |u_{ex}(i) - u_{approx}(i)|= %d',max(abs(u_ex-u2))))
disp(sprintf('  Ltb*: max_i |u_{ex}(i) - u_{approx}(i)|= %d',max(abs(u_ex-u3))))

%relative error
disp('----------------------------------------------');
disp(sprintf('  Mtb:  ||u_{ex}-u_{approx}||_{oo}/||f||_1=%d',norm(u1-u_ex,inf)/norm(f,1)))
disp(sprintf('  Ltb:  ||u_{ex}-u_{approx}||_{oo}/||f||_1=%d',norm(u2-u_ex,inf)/norm(f,1)))
disp(sprintf('  Ltb*: ||u_{ex}-u_{approx}||_{oo}/||f||_1=%d',norm(u3-u_ex,inf)/norm(f,1)))
