% BSFFT - a Matlab toolbox for the butterfly sparse Fourier transform
% 
% Copyright (C) 2012 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2012-08-29 11:15:41 +0200 (Mi, 29. Aug 2012) $
% $Author: melzer $
% $Revision: 3137 $

%START3D test script for d=3
clear all;
addpath('paper_KuMe2012/utilities')

%setting parameters:
%L=3;
L=5;
p=5;
N=2^L;



%nodes on spheres in [0,N]^3
r1=N/2;
r2=2/5*N;
%matrix MX
	M1=N;
	X=randn(M1^2,3);
	nX=repmat(sqrt(sum(X.^2,2)),1,3);
	X=X./nX;
	MX=N/2+r1*X;
	MX=unique(MX,'rows');
	M1=size(MX,1);
	clear X nX;
%matrix MOmega
	M2=N;
	Omega=randn(M2^2,3);
	nOmega=repmat(sqrt(sum(Omega.^2,2)),1,3);
	Omega=Omega./nOmega;
	MOmega=N/2+r2*Omega;
	MOmega=unique(MOmega,'rows');
	M2=size(MOmega,1);
	clear Omega nOmega;

%coefficients
f=ones(M2^2,1);

%plans for different variants

%1. 
disp('Mtb: computation via monomial-type basis')
plan1=sparse_FFT3D(MX,MOmega,p,N,'Mtb'); %monomial type basis
     start1=tic;
 u1=mtimes(plan1,f);
     t1=toc(start1);
 %clear plan1;
 	 
%2. 
disp('Ltb: computation via Lagrange-type basis without precomputation of the polynomials in the last step') 
plan2=sparse_FFT3D(MX,MOmega,p,N,'Ltb'); 
	start2=tic;
u2=mtimes(plan2,f);
	t2=toc(start2);
clear plan2;

%3. 
disp('Ltb*: computation via Lagrange type basis with precomputation of the polynomials in the last step')
plan3=sparse_FFT3D(MX,MOmega,p,N,'Ltb*');
    start3=tic;
u3=mtimes(plan3,f);
	t3=toc(start3);
clear plan3;

%4. 
display('exact computation')
tic
    u_ex=u_exact(MX,MOmega,f,N);
t_ex=toc;


fprintf('\n')
disp(sprintf('test with parameters:'))
disp(sprintf('  N= %d',N))
disp(sprintf('  M1= %d',M1))
disp(sprintf('  M2= %d',M2))
disp(sprintf('  p= %d',p))
disp(sprintf('  time of the exact computation = %d', t_ex))

fprintf('\n')
%time
disp(sprintf('comparison of the three variants:'))
disp(sprintf('  time of Mtb  = %d',t1))
disp(sprintf('  time of Ltb  = %d',t2))
disp(sprintf('  time of Ltb* = %d',t3))
disp('-----------------------------------------------');

%absolute error
disp(sprintf('  Mtb:  max_i |u_{ex}(i) - u_{approx}(i)|= %d',max(abs(u_ex-u1))))
disp(sprintf('  Ltb:  max_i |u_{ex}(i) - u_{approx}(i)|= %d',max(abs(u_ex-u2))))
disp(sprintf('  Ltb*: max_i |u_{ex}(i) - u_{approx}(i)|= %d',max(abs(u_ex-u3))))

%relative error
disp('----------------------------------------------');
disp(sprintf('  Mtb:  ||u_{ex}-u_{approx}||_{oo}/||f||_1=%d',norm(u1-u_ex,inf)/norm(f,1)))
disp(sprintf('  Ltb:  ||u_{ex}-u_{approx}||_{oo}/||f||_1=%d',norm(u2-u_ex,inf)/norm(f,1)))
disp(sprintf('  Ltb*: ||u_{ex}-u_{approx}||_{oo}/||f||_1=%d',norm(u3-u_ex,inf)/norm(f,1)))
